@@ -1,5 +1,5 @@
 #!/bin/sh
-# $Id: autogen.sh 569 2005-09-12 18:47:42Z lathiat $
+# $Id: autogen.sh 1593 2007-12-17 12:36:14Z lennart $
 
 # This file is part of avahi.
 #
@@ -22,7 +22,19 @@ VERSION=1.9
 
 run_versioned() {
     local P
-    type -p "$1-$2" &> /dev/null && P="$1-$2" || local P="$1"
+    local V
+
+    V=$(echo "$2" | sed -e 's,\.,,g')
+    
+    if [ -e "`which $1$V`" ] ; then
+    	P="$1$V" 
+    else
+	if [ -e "`which $1-$2`" ] ; then
+	    P="$1-$2" 
+	else
+	    P="$1"
+	fi
+    fi
 
     shift 2
     "$P" "$@"
@@ -37,10 +49,19 @@ else
     rm -rf autom4te.cache
     rm -f config.cache
 
-    libtoolize -c --force
+    rm -f Makefile.am~ configure.ac~
+    # Evil, evil, evil, evil hack
+    sed 's/read dummy/\#/' `which gettextize` | sh -s -- --copy --force
+    test -f Makefile.am~ && mv Makefile.am~ Makefile.am
+    test -f configure.ac~ && mv configure.ac~ configure.ac
+
+    test "x$LIBTOOLIZE" = "x" && LIBTOOLIZE=libtoolize
+
+    intltoolize --copy --force --automake
+    "$LIBTOOLIZE" -c --force
     run_versioned aclocal "$VERSION" -I common
-    autoconf -Wall
-    autoheader
+    run_versioned autoconf 2.59 -Wall
+    run_versioned autoheader 2.59
     run_versioned automake "$VERSION" -a -c --foreign
 
     if test "x$NOCONFIGURE" = "x"; then
