@@ -1,4 +1,4 @@
-/* $Id: dbus.c 1124 2006-01-30 20:57:58Z lennart $ */
+/* $Id: dbus.c 1266 2006-08-23 23:35:46Z lennart $ */
 
 /***
   This file is part of avahi.
@@ -88,7 +88,8 @@ static const char * const table[- AVAHI_ERR_MAX] = {
 
     AVAHI_DBUS_ERR_NOT_PERMITTED,
     AVAHI_DBUS_ERR_INVALID_ARGUMENT,
-    AVAHI_DBUS_ERR_IS_EMPTY
+    AVAHI_DBUS_ERR_IS_EMPTY,
+    AVAHI_DBUS_ERR_NO_CHANGE
 };
 
 struct error_map {
