@@ -1,21 +1,21 @@
 #ifndef foodefshfoo
 #define foodefshfoo
 
-/* $Id: defs.h 1477 2007-05-09 19:45:54Z lennart $ */
+/* $Id: defs.h 1565 2007-10-28 00:26:15Z lennart $ */
 
 /***
   This file is part of avahi.
- 
+
   avahi is free software; you can redistribute it and/or modify it
   under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of the
   License, or (at your option) any later version.
- 
+
   avahi is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
   Public License for more details.
- 
+
   You should have received a copy of the GNU Lesser General Public
   License along with avahi; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
@@ -27,7 +27,7 @@
 #include <avahi-common/cdecl.h>
 
 /** \mainpage
- *  
+ *
  * \section choose_api Choosing an API
  *
  * Avahi provides three programming APIs for integration of
@@ -35,17 +35,19 @@
  *
  * \li <b>avahi-core</b>: an API for embedding a complete mDNS/DNS-SD stack
  * into your software. This is intended for developers of embedded
- * ampliances only. We dissuade from using this API in normal desktop
+ * appliances only. We dissuade from using this API in normal desktop
  * applications since it is not a good idea to run multiple mDNS
  * stacks simultaneously on the same host.
  * \li <b>the D-Bus API</b>: an extensive D-Bus interface for browsing and
  * registering mDNS/DNS-SD services using avahi-daemon. We recommend
- * to use this API for software written in any language but
- * C. (i.e. Python)
+ * using this API for software written in any language other than
+ * C (e.g. Python).
  * \li <b>avahi-client</b>: a simplifying C wrapper around the D-Bus API. We
- * recommend to use this API in C or C++ progams. The D-Bus internals
+ * recommend using this API in C or C++ progams. The D-Bus internals
  * are hidden completely.
- * 
+ * \li <b>avahi-gobject</b>: an object-oriented C wrapper based on
+ * GLib's GObject. We recommd using this API for GNOME/Gtk programs.
+ *
  * All three APIs are very similar, however avahi-core is the most powerful.
  *
  * In addition to the three APIs described above Avahi supports two
@@ -86,7 +88,7 @@
  * files for all definitions that are part of the Avahi API. Please
  * note that these hidden definitions are considered part of the Avahi
  * API and will stay available in the API in the future.
- * 
+ *
  * \section error_reporting Error Reporting
  *
  * Some notes on the Avahi error handling:
@@ -108,7 +110,7 @@
  *
  * \section event_loop Event Loop Abstraction
  *
- * Avahi uses a simple event loop abstraction laye. A table AvahiPoll
+ * Avahi uses a simple event loop abstraction layer. A table AvahiPoll
  * which contains function pointers for user defined timeout and I/O
  * condition event source implementations needs to be passed to
  * avahi_client_new(). An adapter for this abstraction layer is
@@ -128,7 +130,7 @@
  * be established.
  * - Remove your services when the server enters
  * AVAHI_SERVER_COLLISION or AVAHI_SERVER_REGISTERING state. Your
- * services may no be reachable anymore since the local host name is
+ * services may not be reachable anymore since the local host name is
  * no longer established or is currently in the process of being
  * established.
  * - When registering services, use the following algorithm:
@@ -152,7 +154,7 @@
  * AVAHI_PUBLISH_UPDATE when changing the service name! Renaming a
  * DNS-SD service is identical to deleting and creating a new one, and
  * that's exactly what you should do in that case. First call
- * avahi_entry_group_reset() to remove it and than readd it normally.
+ * avahi_entry_group_reset() to remove it and then read it normally.
  *
  * \section good_browse How to Browse for Services
  *
@@ -183,7 +185,7 @@
  * AvahiClient object in that case and reconnect to the server anew -
  * again with passing AVAHI_CLIENT_NO_FAIL to avahi_client_new().
  *
- * We encourage to implement this in all software where service
+ * We encourage implementing this in all software where service
  * discovery is not an integral part of application. e.g. use it in
  * all kinds of background daemons, but not in software like iChat
  * compatible IM software.
@@ -217,7 +219,7 @@ AVAHI_C_DECL_BEGIN
 
 /** States of a server object */
 typedef enum {
-    AVAHI_SERVER_INVALID,          /**< Invalid state (initial) */ 
+    AVAHI_SERVER_INVALID,          /**< Invalid state (initial) */
     AVAHI_SERVER_REGISTERING,      /**< Host RRs are being registered */
     AVAHI_SERVER_RUNNING,          /**< All host RRs have been established */
     AVAHI_SERVER_COLLISION,        /**< There is a collision with a host RR. All host RRs have been withdrawn, the user should set a new host name via avahi_server_set_host_name() */
@@ -282,7 +284,7 @@ typedef enum {
 typedef enum {
     AVAHI_BROWSER_NEW,               /**< The object is new on the network */
     AVAHI_BROWSER_REMOVE,            /**< The object has been removed from the network */
-    AVAHI_BROWSER_CACHE_EXHAUSTED,   /**< One-time event, to notify the user that all entries from the caches have been send */
+    AVAHI_BROWSER_CACHE_EXHAUSTED,   /**< One-time event, to notify the user that all entries from the caches have been sent */
     AVAHI_BROWSER_ALL_FOR_NOW,       /**< One-time event, to notify the user that more records will probably not show up in the near future, i.e. all cache entries have been read and all static servers been queried */
     AVAHI_BROWSER_FAILURE            /**< Browsing failed due to some reason which can be retrieved using avahi_server_errno()/avahi_client_errno() */
 } AvahiBrowserEvent;
@@ -333,12 +335,12 @@ enum {
     AVAHI_DNS_TYPE_MX = 0x0F,
     AVAHI_DNS_TYPE_TXT = 0x10,
     AVAHI_DNS_TYPE_AAAA = 0x1C,
-    AVAHI_DNS_TYPE_SRV = 0x21,
+    AVAHI_DNS_TYPE_SRV = 0x21
 };
 
 /** DNS record classes, see RFC 1035 */
 enum {
-    AVAHI_DNS_CLASS_IN = 0x01,          /**< Probably the only class we will ever use */
+    AVAHI_DNS_CLASS_IN = 0x01          /**< Probably the only class we will ever use */
 };
 
 /** @} */
