@@ -1,4 +1,4 @@
-/* $Id: dbus-util.c 1357 2006-12-31 16:39:16Z lennart $ */
+/* $Id: dbus-util.c 1482 2007-05-21 17:13:05Z lennart $ */
 
 /***
   This file is part of avahi.
@@ -344,6 +344,9 @@ int avahi_dbus_read_strlst(DBusMessage *m, int idx, AvahiStringList **l) {
         k = (const uint8_t*) "";
         n = 0;
         dbus_message_iter_get_fixed_array(&sub2, &k, &n);
+
+        if (!k)
+            k = (const uint8_t*) "";
             
         strlst = avahi_string_list_add_arbitrary(strlst, k, n);
         
