@@ -1,4 +1,4 @@
-/* $Id: ini-file-parser-test.c 879 2005-10-26 21:27:40Z lennart $ */
+/* $Id: ini-file-parser-test.c 1535 2007-09-06 17:01:24Z lennart $ */
 
 /***
   This file is part of avahi.
@@ -19,6 +19,10 @@
   USA.
 ***/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 
 #include <avahi-common/malloc.h>
