@@ -1,4 +1,4 @@
-/* $Id: ini-file-parser.c 926 2005-11-03 23:36:25Z lennart $ */
+/* $Id: ini-file-parser.c 1533 2007-09-06 16:51:35Z lennart $ */
 
 /***
   This file is part of avahi.
@@ -19,6 +19,10 @@
   USA.
 ***/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
