@@ -1,18 +1,18 @@
-/* $Id: avahi-publish.c 1424 2007-04-14 22:19:52Z lennart $ */
+/* $Id: avahi-publish.c 1598 2007-12-17 15:12:55Z lennart $ */
 
 /***
   This file is part of avahi.
- 
+
   avahi is free software; you can redistribute it and/or modify it
   under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of the
   License, or (at your option) any later version.
- 
+
   avahi is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
   Public License for more details.
- 
+
   You should have received a copy of the GNU Lesser General Public
   License along with avahi; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
@@ -30,18 +30,20 @@
 #include <string.h>
 #include <sys/types.h>
 #include <errno.h>
+#include <locale.h>
 
 #include <avahi-common/simple-watch.h>
 #include <avahi-common/error.h>
 #include <avahi-common/malloc.h>
 #include <avahi-common/alternative.h>
+#include <avahi-common/i18n.h>
 #include <avahi-client/client.h>
 #include <avahi-client/publish.h>
 
 #include "sigint.h"
 
 typedef enum {
-    COMMAND_UNSPEC, 
+    COMMAND_UNSPEC,
     COMMAND_HELP,
     COMMAND_VERSION,
     COMMAND_PUBLISH_SERVICE,
@@ -73,12 +75,12 @@ static void entry_group_callback(AvahiEntryGroup *g, AvahiEntryGroupState state,
 
         case AVAHI_ENTRY_GROUP_ESTABLISHED:
 
-            fprintf(stderr, "Established under name '%s'\n", config->name);
+            fprintf(stderr, _("Established under name '%s'\n"), config->name);
             break;
 
         case AVAHI_ENTRY_GROUP_FAILURE:
-            
-            fprintf(stderr, "Failed to register: %s\n", avahi_strerror(avahi_client_errno(client)));
+
+            fprintf(stderr, _("Failed to register: %s\n"), avahi_strerror(avahi_client_errno(client)));
             break;
 
         case AVAHI_ENTRY_GROUP_COLLISION: {
@@ -91,15 +93,15 @@ static void entry_group_callback(AvahiEntryGroup *g, AvahiEntryGroupState state,
                 n = avahi_alternative_host_name(config->name);
             }
 
-            fprintf(stderr, "Name collision, picking new name '%s'.\n", n);
+            fprintf(stderr, _("Name collision, picking new name '%s'.\n"), n);
             avahi_free(config->name);
             config->name = n;
-            
+
             register_stuff(config);
-                
+
             break;
         }
-            
+
         case AVAHI_ENTRY_GROUP_UNCOMMITED:
         case AVAHI_ENTRY_GROUP_REGISTERING:
             ;
@@ -111,7 +113,7 @@ static int register_stuff(Config *config) {
 
     if (!entry_group) {
         if (!(entry_group = avahi_entry_group_new(client, entry_group_callback, config))) {
-            fprintf(stderr, "Failed to create entry group: %s\n", avahi_strerror(avahi_client_errno(client)));
+            fprintf(stderr, _("Failed to create entry group: %s\n"), avahi_strerror(avahi_client_errno(client)));
             return -1;
         }
     }
@@ -121,23 +123,23 @@ static int register_stuff(Config *config) {
     if (config->command == COMMAND_PUBLISH_ADDRESS) {
 
         if (avahi_entry_group_add_address(entry_group, AVAHI_IF_UNSPEC, AVAHI_PROTO_UNSPEC, 0, config->name, &config->address) < 0) {
-            fprintf(stderr, "Failed to add address: %s\n", avahi_strerror(avahi_client_errno(client)));
+            fprintf(stderr, _("Failed to add address: %s\n"), avahi_strerror(avahi_client_errno(client)));
             return -1;
         }
-        
+
     } else {
         AvahiStringList *i;
-        
+
         assert(config->command == COMMAND_PUBLISH_SERVICE);
 
         if (avahi_entry_group_add_service_strlst(entry_group, AVAHI_IF_UNSPEC, AVAHI_PROTO_UNSPEC, 0, config->name, config->stype, config->domain, config->host, config->port, config->txt) < 0) {
-            fprintf(stderr, "Failed to add service: %s\n", avahi_strerror(avahi_client_errno(client)));
+            fprintf(stderr, _("Failed to add service: %s\n"), avahi_strerror(avahi_client_errno(client)));
             return -1;
         }
 
         for (i = config->subtypes; i; i = i->next)
             if (avahi_entry_group_add_service_subtype(entry_group, AVAHI_IF_UNSPEC, AVAHI_PROTO_UNSPEC, 0, config->name, config->stype, config->domain, (char*) i->text) < 0) {
-                fprintf(stderr, "Failed to add subtype '%s': %s\n", i->text, avahi_strerror(avahi_client_errno(client)));
+                fprintf(stderr, _("Failed to add subtype '%s': %s\n"), i->text, avahi_strerror(avahi_client_errno(client)));
                 return -1;
             }
     }
@@ -149,9 +151,9 @@ static int register_stuff(Config *config) {
 
 static void client_callback(AvahiClient *c, AvahiClientState state, AVAHI_GCC_UNUSED void * userdata) {
     Config *config = userdata;
-    
+
     client = c;
-    
+
     switch (state) {
         case AVAHI_CLIENT_FAILURE:
 
@@ -160,38 +162,38 @@ static void client_callback(AvahiClient *c, AvahiClientState state, AVAHI_GCC_UN
 
                 /* We have been disconnected, so let reconnect */
 
-                fprintf(stderr, "Disconnected, reconnecting ...\n");
+                fprintf(stderr, _("Disconnected, reconnecting ...\n"));
 
                 avahi_client_free(client);
                 client = NULL;
                 entry_group = NULL;
 
                 if (!(client = avahi_client_new(avahi_simple_poll_get(simple_poll), AVAHI_CLIENT_NO_FAIL, client_callback, config, &error))) {
-                    fprintf(stderr, "Failed to create client object: %s\n", avahi_strerror(error));
+                    fprintf(stderr, _("Failed to create client object: %s\n"), avahi_strerror(error));
                     avahi_simple_poll_quit(simple_poll);
                 }
 
             } else {
-                fprintf(stderr, "Client failure, exiting: %s\n", avahi_strerror(avahi_client_errno(c)));
+                fprintf(stderr, _("Client failure, exiting: %s\n"), avahi_strerror(avahi_client_errno(c)));
                 avahi_simple_poll_quit(simple_poll);
             }
 
             break;
-            
+
         case AVAHI_CLIENT_S_RUNNING:
 
             if (register_stuff(config) < 0)
                 avahi_simple_poll_quit(simple_poll);
-            
+
             break;
 
         case AVAHI_CLIENT_S_COLLISION:
 
             if (config->verbose)
-                fprintf(stderr, "Host name conflict\n");
+                fprintf(stderr, _("Host name conflict\n"));
 
             /* Fall through */
-            
+
         case AVAHI_CLIENT_S_REGISTERING:
 
             if (entry_group) {
@@ -199,33 +201,33 @@ static void client_callback(AvahiClient *c, AvahiClientState state, AVAHI_GCC_UN
                 entry_group = NULL;
             }
             break;
-            
+
         case AVAHI_CLIENT_CONNECTING:
-            
+
             if (config->verbose)
-                fprintf(stderr, "Waiting for daemon ...\n");
-            
+                fprintf(stderr, _("Waiting for daemon ...\n"));
+
             break;
-            
+
             ;
     }
 }
 
 static void help(FILE *f, const char *argv0) {
     fprintf(f,
-            "%s [options] %s <name> <type> <port> [<txt ...>]\n"
-            "%s [options] %s <host-name> <address>\n\n"
-            "    -h --help            Show this help\n"
-            "    -V --version         Show version\n"
-            "    -s --service         Publish service\n"
-            "    -a --address         Publish address\n"
-            "    -v --verbose         Enable verbose mode\n"
-            "    -d --domain=DOMAIN   Domain to publish service in\n"
-            "    -H --host=DOMAIN     Host where service resides\n"
-            "       --subtype=SUBTYPE An additional subtype to register this service with\n"
-            "    -f --no-fail         Don't fail if the daemon is not available\n",
-            argv0, strstr(argv0, "service") ? "[-s]" : "-s",
-            argv0, strstr(argv0, "address") ? "[-a]" : "-a");
+            _("%s [options] %s <name> <type> <port> [<txt ...>]\n"
+              "%s [options] %s <host-name> <address>\n\n"
+              "    -h --help            Show this help\n"
+              "    -V --version         Show version\n"
+              "    -s --service         Publish service\n"
+              "    -a --address         Publish address\n"
+              "    -v --verbose         Enable verbose mode\n"
+              "    -d --domain=DOMAIN   Domain to publish service in\n"
+              "    -H --host=DOMAIN     Host where service resides\n"
+              "       --subtype=SUBTYPE An additional subtype to register this service with\n"
+              "    -f --no-fail         Don't fail if the daemon is not available\n"),
+              argv0, strstr(argv0, "service") ? "[-s]" : "-s",
+              argv0, strstr(argv0, "address") ? "[-a]" : "-a");
 }
 
 static int parse_command_line(Config *c, const char *argv0, int argc, char *argv[]) {
@@ -234,7 +236,7 @@ static int parse_command_line(Config *c, const char *argv0, int argc, char *argv
     enum {
         ARG_SUBTYPE = 256
     };
-    
+
     static const struct option long_options[] = {
         { "help",           no_argument,       NULL, 'h' },
         { "version",        no_argument,       NULL, 'V' },
@@ -295,22 +297,22 @@ static int parse_command_line(Config *c, const char *argv0, int argc, char *argv
 
     if (c->command == COMMAND_PUBLISH_ADDRESS) {
         if (optind+2 !=  argc) {
-            fprintf(stderr, "Bad number of arguments\n");
+            fprintf(stderr, _("Bad number of arguments\n"));
             return -1;
         }
 
         avahi_free(c->name);
         c->name = avahi_strdup(argv[optind]);
         avahi_address_parse(argv[optind+1], AVAHI_PROTO_UNSPEC, &c->address);
-        
+
     } else if (c->command == COMMAND_PUBLISH_SERVICE) {
 
         char *e;
         long int p;
         int i;
-        
+
         if (optind+3 > argc) {
-            fprintf(stderr, "Bad number of arguments\n");
+            fprintf(stderr, _("Bad number of arguments\n"));
             return -1;
         }
 
@@ -321,16 +323,16 @@ static int parse_command_line(Config *c, const char *argv0, int argc, char *argv
         p = strtol(argv[optind+2], &e, 0);
 
         if (errno != 0 || *e || p < 0 || p > 0xFFFF) {
-            fprintf(stderr, "Failed to parse port number: %s\n", argv[optind+2]);
+            fprintf(stderr, _("Failed to parse port number: %s\n"), argv[optind+2]);
             return -1;
         }
 
         c->port = p;
-            
+
         for (i = optind+3; i < argc; i++)
             c->txt = avahi_string_list_add(c->txt, argv[i]);
     }
-        
+
     return 0;
 }
 
@@ -339,6 +341,9 @@ int main(int argc, char *argv[]) {
     Config config;
     const char *argv0;
 
+    avahi_init_i18n();
+    setlocale(LC_ALL, "");
+
     if ((argv0 = strrchr(argv[0], '/')))
         argv0++;
     else
@@ -350,14 +355,14 @@ int main(int argc, char *argv[]) {
     switch (config.command) {
         case COMMAND_UNSPEC:
             ret = 1;
-            fprintf(stderr, "No command specified.\n");
+            fprintf(stderr, _("No command specified.\n"));
             break;
-            
+
         case COMMAND_HELP:
             help(stdout, argv0);
             ret = 0;
             break;
-            
+
         case COMMAND_VERSION:
             printf("%s "PACKAGE_VERSION"\n", argv0);
             ret = 0;
@@ -365,17 +370,17 @@ int main(int argc, char *argv[]) {
 
         case COMMAND_PUBLISH_SERVICE:
         case COMMAND_PUBLISH_ADDRESS:
-            
+
             if (!(simple_poll = avahi_simple_poll_new())) {
-                fprintf(stderr, "Failed to create simple poll object.\n");
+                fprintf(stderr, _("Failed to create simple poll object.\n"));
                 goto fail;
             }
-            
+
             if (sigint_install(simple_poll) < 0)
                 goto fail;
-            
+
             if (!(client = avahi_client_new(avahi_simple_poll_get(simple_poll), config.no_fail ? AVAHI_CLIENT_NO_FAIL : 0, client_callback, &config, &error))) {
-                fprintf(stderr, "Failed to create client object: %s\n", avahi_strerror(error));
+                fprintf(stderr, _("Failed to create client object: %s\n"), avahi_strerror(error));
                 goto fail;
             }
 
@@ -383,30 +388,30 @@ int main(int argc, char *argv[]) {
                 const char *version, *hn;
 
                 if (!(version = avahi_client_get_version_string(client))) {
-                    fprintf(stderr, "Failed to query version string: %s\n", avahi_strerror(avahi_client_errno(client)));
+                    fprintf(stderr, _("Failed to query version string: %s\n"), avahi_strerror(avahi_client_errno(client)));
                     goto fail;
                 }
 
                 if (!(hn = avahi_client_get_host_name_fqdn(client))) {
-                    fprintf(stderr, "Failed to query host name: %s\n", avahi_strerror(avahi_client_errno(client)));
+                    fprintf(stderr, _("Failed to query host name: %s\n"), avahi_strerror(avahi_client_errno(client)));
                     goto fail;
                 }
-                
-                fprintf(stderr, "Server version: %s; Host name: %s\n", version, hn);
+
+                fprintf(stderr, _("Server version: %s; Host name: %s\n"), version, hn);
             }
 
             avahi_simple_poll_loop(simple_poll);
             ret = 0;
             break;
     }
-    
+
 fail:
 
     if (client)
         avahi_client_free(client);
 
     sigint_uninstall();
-    
+
     if (simple_poll)
         avahi_simple_poll_free(simple_poll);
 
